@@ -5,6 +5,7 @@ use Moo;
 use JSON;
 use URI::Escape;
 use REST::Client;
+
 #use Data::Dumper;
 use Carp;
 use Log::Any;
@@ -16,11 +17,11 @@ REST::Client::CrossRef - Read data from CrossRef using its REST API
 
 =cut
 
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
 =head1 VERSION
 
-Version 0.001
+Version 0.002
 
 =cut
 
@@ -181,10 +182,10 @@ has rows => (
     default => sub {0},
     isa     => sub { croak "rows must be under 1000" unless $_[0] < 1000 }
 );
-has code          => ( is => 'rw' );
-has sleep         => ( is => 'rw', default => sub {0} );
-has log           => ( is => 'lazy' );
-has client        => ( is => 'lazy' );
+has code   => ( is => 'rw' );
+has sleep  => ( is => 'rw', default => sub {0} );
+has log    => ( is => 'lazy' );
+has client => ( is => 'lazy' );
 
 =head2  C<$cr = REST::Client::CrossRef-E<gt>new( ... mailto =E<gt> your@email.here, ...)>
 
@@ -193,7 +194,7 @@ See L<https://github.com/CrossRef/rest-api-doc#good-manners--more-reliable-servi
 
 =cut
 
-has mailto        => ( is => 'lazy', default => sub {0});
+has mailto => ( is => 'lazy', default => sub {0} );
 
 =head2 C<$cr = REST::Client::CrossRef-E<gt>new( ... sort_output =E<gt>1, ...)>
 
@@ -203,8 +204,8 @@ In effect only if spit_raw_data is false.
 
 =cut
 
-has sort_output   => ( is => 'lazy', default => sub {0} );
-has test_data     => ( is => 'lazy', default => sub{0} );
+has sort_output => ( is => 'lazy', default => sub {0} );
+has test_data   => ( is => 'lazy', default => sub {0} );
 
 =head2 C<$cr = REST::Client::CrossRef-E<gt>new( ... spit_raw_data =E<gt>1, ...)>
 
@@ -216,7 +217,7 @@ Default to 0.
 
 has spit_raw_data => ( is => 'lazy', default => sub {0} );
 has cursor        => ( is => 'rw' );
-has page_start_at => ( is => 'rw', default => sub {0} );
+has page_start_at => ( is => 'rw',   default => sub {0} );
 
 =head2 C<$cr = REST::Client::CrossRef-E<gt>new( ... add_end_flag =E<gt>1, ...)>
 
@@ -227,7 +228,7 @@ In effect only if spit_raw_data is false.
 
 =cut
 
-has add_end_flag  => ( is => 'lazy', default => sub {1} );
+has add_end_flag => ( is => 'lazy', default => sub {1} );
 
 =head2 C<$cr = REST::Client::CrossRef-E<gt>new( ... keys_to_keep =E<gt> [[key1, key1a, ...], [key2], ... ], ...)>
 
@@ -238,7 +239,7 @@ In effect only if spit_raw_data is false.
 
 =cut
 
-has keys_to_keep  => ( is => 'lazy');
+has keys_to_keep => ( is => 'lazy' );
 
 =head2 C<$cr = REST::Client::CrossRef-E<gt>new( ... version =E<gt> "v1", ... )>
 
@@ -250,7 +251,6 @@ See L<https://github.com/CrossRef/rest-api-doc#api-versioning>
 
 has version => ( is => 'ro' );
 
-
 sub _build_client {
     my ($self) = @_;
     my $client = REST::Client->new();
@@ -273,7 +273,6 @@ sub _build_client {
     $client;
 }
 
-
 sub _build_log {
     my ($self) = @_;
     Log::Any->get_logger( category => ref($self) );
@@ -345,15 +344,16 @@ sub _crossref_get_request {
     if ($query_ar) {
 
         for my $p (@$query_ar) {
-            print "$p\n";
+            #print "$p\n";
             push @params, $p;
 
         }
     }
     for my $name ( keys %param ) {
         my $value = $param{$name};
-        #location:United Kingdom space is uri_escape twice 
-        #push @params, uri_escape($name) . "=" . uri_escape($value) if ($value);
+
+      #location:United Kingdom space is uri_escape twice
+      #push @params, uri_escape($name) . "=" . uri_escape($value) if ($value);
         push @params, $name . "=" . $value if ($value);
 
     }
@@ -395,31 +395,32 @@ sub _crossref_get_request {
     }
     elsif ( $code eq '429' || $code eq '503' ) {
         $self->sleep( $retryAfter // 60 );
-        return undef;
+        return;
     }
 
     $self->log->notice( "> Content: " . $response->responseContent );
 
     $self->code($code);
 
-    return undef unless $code eq '200';
+    return unless $code eq '200';
 
     $response;
 }
 
 sub _get_metadata {
     my ( $self, $path, $query_ar, $filter, $select ) = @_;
-      $self->log->notice("test_data: ", ($self->test_data() ? " 1 ": " 0 "));
+    $self->log->notice( "test_data: ",
+        ( $self->test_data() ? " 1 " : " 0 " ) );
     $self->page_start_at(0);
     my $response =
         $self->_crossref_get_request( $path,
         $query_ar, ( filter => $filter, select => $select ) );
 
     #   print Dumper $response;
-    return undef unless $response;
+    return unless $response;
 
     #my $hr        = decode_json $response->responseContent;
-  
+
     my $hr =
         $self->test_data()
         ? decode_json( $self->test_data() )
@@ -511,17 +512,22 @@ sub _display_data {
         my @data;
 
         for my $data_hr (@$data_ar) {
+
             #my $result_hr={};
             # my $result_hr;
             # $self->_unfold_hash( $data_hr, $keys_ar, $result_hr );
 
-             my $formatter = REST::Client::CrossRef::Unfolder->new;
-             $formatter->set_keys_to_keep($self->{keys_to_keep}) if $self->{keys_to_keep};
-             my $result_hr = {};
-             $formatter->_unfold_hash($data_hr, undef, $result_hr);
-             
-             #  $self->log->debug("display_data\n", Dumper $result_hr);
-             
+            my $formatter = REST::Client::CrossRef::Unfolder->new;
+            $formatter->set_keys_to_keep( $self->{keys_to_keep} )
+                if $self->{keys_to_keep};
+
+            #https://www.perlmonks.org/?node_id=1224994
+            #my $result_hr = {};
+            #$formatter->_unfold_hash($data_hr, undef, $result_hr);
+            my $result_hr = $formatter->_unfold_hash($data_hr);
+
+            #  $self->log->debug("display_data\n", Dumper $result_hr);
+
             if ( $self->{keys_to_keep} ) {
 
                 # die;
@@ -539,9 +545,12 @@ sub _display_data {
                     # print "pos undef for $key\n" unless defined $pos;
                     #$key_unseen{$pos}= $key;
                     $pos_seen{$pos} = 1;
-                    my $value =  (defined $result_hr->{$key}) ? $result_hr->{$key} : "";
-                    $self->log->debug($key, " - ", $value);
-                    $item_data[$pos] = {$key => $value};
+                    my $value =
+                        ( defined $result_hr->{$key} )
+                        ? $result_hr->{$key}
+                        : "";
+                    $self->log->debug( $key, " - ", $value );
+                    $item_data[$pos] = { $key => $value };
 
                 }
 
@@ -585,7 +594,6 @@ sub _display_data {
     }
 }
 
-
 =head2 C<$cr-E<gt>rows( $row_value )>
 
 Set the rows parameter that determines how many items are returned in one page
@@ -674,8 +682,9 @@ sub articles_from_funder {
                   $href->{$k} =~ /^https*:\/\/orcid.org\//
                 ? $href->{$k}
                 : "http://orcid.org/" . $href->{$k};
-                # $self->{param} = "orcid:" . uri_escape($url);
-                $self->{filter} = "orcid:" . uri_escape($url);
+
+            # $self->{param} = "orcid:" . uri_escape($url);
+            $self->{filter} = "orcid:" . uri_escape($url);
             return $self->_get_metadata( "/funders/$id/works", undef,
                 $self->{filter}, $self->{select} );
         }
@@ -738,7 +747,7 @@ Retrieve all the metadata (> 60'000 items) from the journals road.
 sub get_journals {
     my $self = shift;
     $self->{path} = "/journals";
-     $self->page_start_at(1);
+    $self->page_start_at(1);
     $self->_get_page_metadata("/journals");
 
 }
@@ -790,7 +799,7 @@ sub query_works {
 
 =head2 C<$cr-E<gt>query_articles( $fields_array_ref, $values_array_ref )>
 
-A shortcut for C<$cr-&lt;query_works($fields_array_ref, $values_array_ref,  "title,container-title,page,issued,volume,issue,author,published-print,published-online")>
+A shortcut for C<$cr-E<gt>query_works($fields_array_ref, $values_array_ref,  "title,container-title,page,issued,volume,issue,author,published-print,published-online")>
 
 =cut
 
@@ -803,7 +812,7 @@ sub query_articles {
 
 =head2 C<$cr-E<gt>query_journals( $fields_array_ref, $values_array_ref )>
 
-A shortcut for C<$cr-&lt;query_works($fields_array_ref, $values_array_ref, "container-title,page,issued,volume,issue">
+A shortcut for C<$cr-E<gt>query_works($fields_array_ref, $values_array_ref, "container-title,page,issued,volume,issue">
 
 =cut
 
@@ -824,12 +833,14 @@ Return undef after the last set.
 sub get_next {
     my $self = shift;
     $self->log->debug( "get_next cursor: ",
-        ( defined $self->cursor ? " defined " : " undef" ));
-    $self->log->debug("get_next page_start_at: ", $self->page_start_at);
+        ( defined $self->cursor ? " defined " : " undef" ) );
+    $self->log->debug( "get_next page_start_at: ", $self->page_start_at );
 
     if ( $self->cursor ) {
-        $self->_get_metadata( $self->{path}, $self->{param}, $self->{filter},
-            $self->{select} );
+        $self->_get_metadata(
+            $self->{path},   $self->{param},
+            $self->{filter}, $self->{select}
+        );
     }
     my $last_start = $self->page_start_at;
 
@@ -924,40 +935,49 @@ use Carp;
 use Log::Any;
 use Moo;
 
-extends 'REST::Client::CrossRef'; 
+extends 'REST::Client::CrossRef';
+
 #=for comment
- has 'log' => (
-        is => 'ro',
-        default => sub { Log::Any->get_logger(  category => "unfolder") },
-    );
+has 'log' => (
+    is      => 'ro',
+    default => sub { Log::Any->get_logger( category => "unfolder" ) },
+);
+
 #=cut
- # This setting of the array ref could be removed since the ordering in display_data
- # also remove the keys that are not wanted. But the hash builded is smaller
- # with adding only the key that are needed.
+# This setting of the array ref could be removed since the ordering in display_data
+# also remove the keys that are not wanted. But the hash builded is smaller
+# with adding only the key that are needed.
 sub set_keys_to_keep {
-    my ($self, $ar_ref) = @_;
+    my ( $self, $ar_ref ) = @_;
     $self->{keys_to_keep} = $ar_ref;
 
 }
+
 sub _unfold_hash {
     my ( $self, $raw_hr, $key_ar, $result_hr ) = @_;
 
-     $self->log->debug( "unfold_hash1: ", ($result_hr ? scalar %$result_hr : 0));
+    $self->log->debug( "unfold_hash1: ",
+        ( $result_hr ? scalar %$result_hr : 0 ) );
     for my $k ( keys %$raw_hr ) {
+
         # $self->log->debug( "key: ", $k );
 
         push @$key_ar, $k;
 
         if ( ref $raw_hr->{$k} eq "HASH" ) {
 
-            $self->_unfold_hash( $raw_hr->{$k}, $key_ar, $result_hr );
+            $result_hr =
+                $self->_unfold_hash( $raw_hr->{$k}, $key_ar, $result_hr );
 
-            $self->log->debug("1 size ", $result_hr ? scalar %$result_hr : 0);
+            $self->log->debug( "1 size ",
+                $result_hr ? scalar %$result_hr : 0 );
         }
         elsif ( ref $raw_hr->{$k} eq "ARRAY" ) {
-           $self->_unfold_array( $raw_hr->{$k}, $key_ar, $result_hr );
+            $result_hr =
+                $self->_unfold_array( $raw_hr->{$k}, $key_ar, $result_hr );
 
-            $self->log->debug("2 size ", $result_hr ? scalar %$result_hr : 0);
+            $self->log->debug( "2 size ",
+                $result_hr ? scalar %$result_hr : 0 );
 
             $result_hr->{ $key_ar->[$#$key_ar] } =~ s/,\s$//
                 if ( defined $result_hr->{ $key_ar->[$#$key_ar] } );
@@ -966,59 +986,70 @@ sub _unfold_hash {
 
         else {
 
-            $self->log->debug( "ref: ", ref $raw_hr->{$k}) if (ref  $raw_hr->{$k});
+            $self->log->debug( "ref: ", ref $raw_hr->{$k} )
+                if ( ref $raw_hr->{$k} );
             my $key = join( "/", @$key_ar );
 
+=for comment
              if ( $self->{keys_to_keep} ) {
                  $self->log->debug("key with key_to_keep set : ", $key, " value: ",  $raw_hr->{$k} );
                 $result_hr->{$key} = $raw_hr->{$k}
                     if ( defined $self->{keys_to_keep}->{$key} );
             }
+=cut
+
+            if (   defined $self->{keys_to_keep}
+                && defined $self->{keys_to_keep}->{$key} )
+            {
+                $result_hr->{$key} = $raw_hr->{$k}
+
+            }
             else {
-                 $self->log->debug("key : ", $key, " value: ",  $raw_hr->{$k});
+                $self->log->debug( "key : ", $key, " value: ",
+                    $raw_hr->{$k} );
                 $result_hr->{$key} = $raw_hr->{$k};
             }
 
-           }
+        }
 
-           # $result_hr->{$key} = $raw_hr->{$k};
+        # $result_hr->{$key} = $raw_hr->{$k};
         my $tmp = pop @$key_ar;
 
         #   print " removed ", $tmp, "\n";
 
     }
 
-    $self->log->debug("_unfold_hash3: ", $result_hr ? scalar (%$result_hr) : 0);
-    # return $result_hr;
+    $self->log->debug( "_unfold_hash3: ",
+        $result_hr ? scalar(%$result_hr) : 0 );
+    return $result_hr;
 }
 
 sub _unfold_array {
     my ( $self, $ar, $key_ar, $res_hr ) = @_;
 
-    $self->log->debug("_unfold_array0: ", $res_hr ? scalar (%$res_hr) : 0);
+    $self->log->debug( "_unfold_array0: ", $res_hr ? scalar(%$res_hr) : 0 );
     my $last_key = join( "/", @{$key_ar} );
     my $key = $key_ar->[$#$key_ar];
 
-    $self->log->debug( "_unfold array1 key: ", $key);
+    $self->log->debug( "_unfold array1 key: ", $key );
     if ( $key eq "author" ) {
         my $first;
         my @all;
-        for $a (@$ar) {
-            if ( $a->{sequence} eq 'first' ) {
+        for my $aut (@$ar) {
+            if ( $aut->{sequence} eq 'first' ) {
                 $first =
-                $a->{family} . ( defined $a->{given} ? ", ". $a->{given} : " "). $self->_unfold_affiliation( $a->{affiliation} );
-                #$a->{family} . ", "  . $a->{given}  . $self->_unfold_affiliation( $a->{affiliation} );
-
-                # $first.= $self->_unfold_affiliation( $a->{affiliation} );
-                #print "first: ", $first, "\n";
-                #my $tmp = $self->_unfold_affiliation( $a->{affiliation} );
-                #print  "aff: ", $tmp, "\n";
-                #die Dumper( $first . $tmp);
+                      $aut->{family}
+                    . ( defined $aut->{given} ? ", " . $aut->{given} : " " )
+                    . $self->_unfold_affiliation( $aut->{affiliation} );
 
             }
             else {
-                #push @all, "\n" . $a->{family} . ", "  . $a->{given} . $self->_unfold_affiliation( $a->{affiliation} );
-                push @all, "\n" . $a->{family} . ( defined $a->{given} ?  ", "  . $a->{given} : " ") . $self->_unfold_affiliation( $a->{affiliation} );
+#push @all, "\n" . $a->{family} . ", "  . $a->{given} . $self->_unfold_affiliation( $a->{affiliation} );
+                push @all,
+                      "\n"
+                    . $aut->{family}
+                    . ( defined $aut->{given} ? ", " . $aut->{given} : " " )
+                    . $self->_unfold_affiliation( $aut->{affiliation} );
             }
 
         }
@@ -1032,21 +1063,24 @@ sub _unfold_array {
         for my $val (@$ar) {
 
             if ( ref $val eq "HASH" ) {
-                $self->_unfold_hash( $val, $key_ar, $res_hr );
+                $res_hr = $self->_unfold_hash( $val, $key_ar, $res_hr );
                 my $last = $#$key_ar;
                 $res_hr->{ $key_ar->[$last] } =~ s/,\s$//
                     if ( defined $res_hr->{ $key_ar->[$last] } );
 
-          $self->log->debug( "_unfold_array2: ", $res_hr ? scalar (%$res_hr) : 0);
+                $self->log->debug( "_unfold_array2: ",
+                    $res_hr ? scalar(%$res_hr) : 0 );
             }
             elsif ( ref $val eq "ARRAY" ) {
-                  $self->_unfold_array( $val, $key_ar, $res_hr );
+                $res_hr = $self->_unfold_array( $val, $key_ar, $res_hr );
 
-           $self->log->debug("_unfold_array3: ", $res_hr ? scalar (%$res_hr) : 0);
+                $self->log->debug( "_unfold_array3: ",
+                    $res_hr ? scalar(%$res_hr) : 0 );
 
             }
             else {
 
+=for comment
                 if ( $self->{keys_to_keep} ) {
                     return unless defined $self->{keys_to_keep}->{$last_key};
 
@@ -1058,6 +1092,19 @@ sub _unfold_array {
                         $res_hr->{$last_key} = "";
                     }
                 }
+=cut
+
+                if (   defined $self->{keys_to_keep}
+                    && defined $self->{keys_to_keep}->{$last_key} )
+                {
+                    if ( defined $val ) {
+                        $res_hr->{$last_key} .= $val . ", ";
+                    }
+                    else {
+                        $res_hr->{$last_key} = "";
+                    }
+
+                }
                 else {
                     # print "key3: ", $key, "\n";
                     $res_hr->{$last_key} .= $val . ", ";
@@ -1068,8 +1115,8 @@ sub _unfold_array {
 
     }
 
-    $self->log->debug("_unfold_array4: ", $res_hr ? scalar (%$res_hr) : 0);
-    #return $res_hr;
+    $self->log->debug( "_unfold_array4: ", $res_hr ? scalar(%$res_hr) : 0 );
+    return $res_hr;
 }
 
 sub _unfold_affiliation {
@@ -1124,9 +1171,8 @@ L<http://sourceforge.net/projects/rest-client-crossref/>
 
 =head1 AUTHOR
 
-    FranE<ccedil>ois Rappaz
-    CPAN ID: RAPPAZF
-    rappazf@gmail.com
+    F. Rappaz
+    CPAN ID: RAPPAZF 
 
 =head1 COPYRIGHT
 
